function [z, Z] = rjmcmcot(X, n0, I, priorr, da, t, seed)
%
%RJMCMC --Reversible Jump Markov Chain Monte Carlo used for identifying
%       expressed genes for SAGE library data analysis.  
%       The data is in X with Gxn matrix where G is the number of total
%       gene and n=sum(n0) is the number of individuals. 
%
%       The data has K>=2 groups by individuals where K=length(n0).
%       n0 contains the library numbers for each group.
%
% This version considers move ``one'' gene after updating parameters.
% We assume p_{kg}~\beta(t\alpha_{kg}, t(\bar{N_k}-\alpha_{kg})),
% k=1,...,K.
%
% [z, Z] = rjmcmcot(X, n0, I, priorr, da, t, seed) calculate the
% posterior probability for each tag falling in the DE (differential 
% expressed) set. All input arguments have default values. But if you give 
% the first argument X, you have to input arguments n0 and I.
%
%     X:(sagedata), Gxn matrix for SAGE library data.
%     n0:([3,3]), 1xK vector gives the number of libraries for each froup.
%     I:(80000), Total iteration numbers.
%     priorr:(0.1), prior probability ratio for a tag in S1.
%     da:(2), the parameter for updating alpha.
%     t:(1), prior sensitivity parameter.
%     seed:(1234), random seed or state.
%--------------------------------------------------------------------------
%Output arguments:
%    z: Gx3 matrix showing: column 1, tag index
%                           column 2, posterior probability
%                           column 3, Bayes Factor.
%    Z: Ix5 matrix showing: column 1, iteration number
%                           column 2, the tag g being picked
%                           column 3, the group from which g is picked
%                           column 4, the group g is going to
%                           column 5, adding(1) or deleting(-1) or no moving(0). 
% References:
%
%   [1] Wang, Z. and Lin, S. Modeling and Analysis of SAGE Libraries,
%       manuscript at http://www.stat.ohio-state.edu/~statgen/DE-
%       SAGE.html, 2005.

%Author: Zailong Wang & Shili Lin, MBI at The Ohio State University
%$Revision: 1.5 $  $Date: 2005/10/12 21:49:12 $

if nargin < 7, seed = 1234; end %random seed or state.
if nargin < 6, t=1; end %prior sensitivity parameter.
if nargin < 5, da=2; end %used in q(alpha1|alpha0)~U[alpha0/da, alpha0*da];
if nargin < 4, priorr=0.1; end %prior ratio.
if nargin == 2 | nargin == 1, 
        error('stats:rimcmcot: Too Few Inputs',...
        'At least three input arguments required.');
end

if nargin < 1, X=load('sagedata.txt'); n0=[3,3]; I=80000; end
%--------------------------------------------------------------%

initstate(seed);
X=X+0.1;
%add epsi=0.1 to X to chang 0 to 0.01 to avoid log(0)=-infty;
%-------------------------------------------------------------------------%

[G, n]=size(X); K=length(n0); K1=K-1;
N=sum(X); sumN=sum(N); Nbar=mean(N); 
XN=N(ones(1,G), :); P=X./XN;

minp=min(min(X./N(ones(1, G),:))); %minimum of p value for whole algorithm.
X=[X, [1:G]']; n_G1=floor(G/2); n_G2=G-n_G1;
%n_G1/2 is the number of genes (tags) in Class 1/2;
gindex=randperm(G); gindex1=gindex(1:n_G1); gindex2=gindex((n_G1+1):G);
XG1=X(gindex1, :); XG2=X(gindex2, :);
G1=zeros(1, I); G2=zeros(1, I); %gene g in class 1/2 at iteration i;
Z1=zeros(I+1, 3); Z1(1,:)=[0, n_G1, n_G2];%keep the gene numbers for XG1/2;
Z2=zeros(I, 5); %keep which gene in which group at every iteration.     

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Initial parameter values for XG1         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
b=n; b1=n0; N1=cell(1, K); X1=cell(1, K); a1g=[]; p1g0=[]; alpha1g0=[];
sumN1=[]; N1bar=[]; n1=1; n2=n0(1); 
for k=1:K
    N1(1,k)={N(n1:n2)}; sumN1(k)=sum(N1{1,k}); N1bar(k)=mean(N1{1,k});
    X1(1,k)={XG1(:, n1:n2)}; a1g(:,k)=sum(X1{1,k}, 2); 
    p1g0(:,k)=a1g(:,k)/sumN1(k); 
    %initial alpha1g called alpha1g0;
      for j=1:n_G1
      alpha1g0(j,k)=N1bar(k);
      while alpha1g0(j,k) >= N1bar(k);
          alpha1g0(j,k)=gamrnd(a1g(j,k), 1/b1(k));
      end
      end
    %end initial alpha1g; 
    if k<K; n1=n1+n0(k); n2=n2+n0(k+1); end;
end
clear i j gindex gindex1 gindex2 k n1 n2;
X2=XG2(:, 1:n);  ag=sum(X2,2); pg0=ag./sumN; 
    %initial alphag called alphg0;
      alphag0=ones(n_G2,1)*Nbar;
      for j=1:n_G2
        while alphag0(j) >= Nbar;
          alphag0(j)=gamrnd(ag(j), 1/b);
        end
      end
    %end initial alpha1g; 

for i=1:I
    i
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %updating the parameter values;%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if n_G1>0 %updating parameters for XG1;
        n1=1; n2=n0(1); alpha1g1=alpha1g0; p1g1=p1g0;
        for k=1:K
               %updating alphag by Metropolis-Hastings--------------------;
               lalpha1g=alpha1g0(:,k)/da;
               ralpha1g=min(N1bar(k), da*alpha1g0(:,k));
               alpha1g=unifrnd(lalpha1g, ralpha1g);

            rg1=gammaln(alpha1g0(:,k)) - gammaln(alpha1g);
            rg2=gammaln(N1bar(k)-alpha1g0(:,k))-gammaln(N1bar(k)-alpha1g);
            rg3=(a1g(:,k)-1).*(log(alpha1g)-log(alpha1g0(:,k)));
            rg4=(alpha1g-alpha1g0(:,k)).*(log(p1g0(:,k))-b1(k)...
                -log(1-p1g0(:,k)));
            rg5=(da*min(N1bar(k), da*alpha1g0(:,k))-alpha1g0(:,k))...
                ./(da*min(N1bar(k), da*alpha1g)-alpha1g);
            rg0=exp(rg1+rg2+rg3+rg4).*rg5;
            for j = 1:n_G1;
                if rg0(j)>rand; alpha1g1(j,k)=alpha1g(j); end
            end
            clear lalpha1g ralpha1g alpha1g rg1 rg2 rg3 rg4 rg5 rg0 j;

            %updating pg by Metropolis-Hastings---------------------------;
            p1g=betarnd(2, 2./p1g0(:,k)-2);
            rg1=log(2-p1g)-log(2-p1g0(:,k));
            rg2=(t*alpha1g1(:,k)+a1g(:,k)-4).*(log(p1g)-log(p1g0(:,k)));
            rg3=(t*(N1bar(k)-alpha1g1(:,k))+3-2./p1g0(:,k)).*log(1-p1g);
            rg4=(t*(N1bar(k)-alpha1g1(:,k))+3-2./p1g).*log(1-p1g0(:,k));
            rg5=sumN1(k).*(p1g-p1g0(:,k));
            rg0=exp(rg1+rg2+rg3-rg4-rg5);
            for j = 1:n_G1;
                if rg0(j)>rand; p1g1(j,k)=p1g(j); end
            end
            clear p1g rg1 rg2 rg3 rg4 rg5 rg0 j;
            if k<K; n1=n1+n0(k); n2=n2+n0(k+1); end;
        end %END of (for k=1:K)
       end %end of if n_G1>0;
       %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
       
       if n_G2>0 %updating parameters for XG2;
            alphag1=alphag0; pg1=pg0;
            
            %updating alphag by Metropolis-Hastings--------------------;
                lalphag=alphag0/da; 
                ralphag=min(Nbar, da*alphag0);
                alphag=unifrnd(lalphag, ralphag);

            rg1=gammaln(alphag0) - gammaln(alphag);
            rg2=gammaln(Nbar-alphag0)- gammaln(Nbar-alphag);
            rg3=(ag-1).*(log(alphag)-log(alphag0));
            rg4=(alphag-alphag0).*(log(pg0)-b-log(1-pg0));
            rg5=(da*min(Nbar, da*alphag0)-alphag0)...
                ./(da*min(Nbar, da*alphag)-alphag);
            rg0=exp(rg1+rg2+rg3+rg4).*rg5;
            for j = 1:n_G2;
                if rg0(j)>rand; alphag1(j)=alphag(j); end
            end
            clear lalphag ralphag alphag rg1 rg2 rg3 rg4 rg5 rg0 j;
            
            %updating pg by Metropolis-Hastings---------------------------;
            pg=betarnd(2, 2./pg0-2);
            rg1=log(2-pg)-log(2-pg0);
            rg2=(t*alphag1+ag-4).*(log(pg)-log(pg0));
            rg3=(t*(Nbar-alphag1)+3-2./pg0).*log(1-pg);
            rg4=(t*(Nbar-alphag1)+3-2./pg).*log(1-pg0);
            rg5=sumN*(pg-pg0);
            rg0=exp(rg1+rg2+rg3-rg4-rg5);
            for j = 1:n_G2;
                if rg0(j)>rand; pg1(j)=pg(j); end
            end
            clear pg rg1 rg2 rg3 rg4 rg5 rg0 j;
       end %end of if n_G2>0;
%--------------------------------------------------------------------------
       %end of updating parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  if isempty(XG1) %Choosing which group to operate (add or delete)                                          %
        group=2;                                             %
    elseif isempty(XG2)                                      %
        group=1;                                             %
    else if rand < n_G1/G                                    %
            group=1;                                         %
        else group=2;                                        %
        end                                                  %
   end %------------if isempty(XG1)----------------%%%%%%%%%%%
    
    if group==1 %deleting from XG1?
        g0=randsample(n_G1, 1); %one gene for deleting;   
        Xg=XG1(g0,:); g=Xg(n+1); g0alpha1=alpha1g1(g0,:); g0p1=p1g1(g0,:);  
        
        % Calculating Acceptance Probability: ap
        %----------------------------------------------------------------
        g0p=mean(g0p1); g0alpha=Nbar*sum(g0alpha1./N1bar)/K;
        g0a1=a1g(g0, :); g0a=sum(g0a1); lambda1g=g0a1./b1;
        
        u1g=ones(1,K); v1g=ones(1,K);
        ibetax1=min(1, K*g0p/K1); 
        ibetax2=max(0, (K*g0p-1)/K1);
        igammax1=min(N1bar, N1bar*(K*g0alpha/(K1*Nbar)));
        igammax2=max(0, N1bar*((K*g0alpha/Nbar-1)/K1));

        for k1=1:K
            u1g(k1)=ibetax1;
          while u1g(k1) >= ibetax1 | u1g(k1) <= ibetax2; 
             u1g(k1)=betarnd(lambda1g(k1), N1bar(k1)-lambda1g(k1));
          end;
            v1g(k1)=igammax1(k1);
          while v1g(k1) >= igammax1(k1) | v1g(k1) <= igammax2(k1) 
             v1g(k1)=gamrnd(g0a1(k1), 1./b1(k1));     
          end;
        end
        
      ap11=sum(g0a1.*log(g0p1))-g0a*log(g0p);
      ap12=sumN*g0p-sum(sumN1.*g0p1);
      ap1=2*log(K)+ap11+ap12;

      ap21=log(N1bar(K))+sum(gammaln(t*N1bar))+gammaln(g0a);
      ap22=gammaln(t*g0alpha)+gammaln(t*(Nbar-g0alpha))+...
            log(gammainc(b*Nbar, g0a));
      ap23=gammaln(lambda1g)+gammaln(N1bar-lambda1g);
      ap24=log(Nbar)+gammaln(t*Nbar)+gammaln(g0a1(K));
      ap25=gammaln(t*g0alpha1)+gammaln(t*(N1bar-g0alpha1))+...
           log(gammainc(b1.*N1bar, g0a1));
      ap26=gammaln(N1bar(1:K1));
      ap2=ap21+ap22+sum(ap23(1:K1))-ap24-sum(ap25)-sum(ap26);
      
      ap31=g0a1(K)*log(b1(K));
      ap32=(g0a1-1).*log(g0alpha1)+(t*g0alpha1-1).*log(g0p1);
      ap33=(t*(N1bar-g0alpha1)-1).*log(1-g0p1);
      ap34=g0a*log(b)+(g0a-1)*log(g0alpha);
      ap35=(t*g0alpha-1)*log(g0p)+(t*(Nbar-g0alpha)-1)*log(1-g0p);
      ap36=(lambda1g-1).*log(u1g)+(N1bar-lambda1g-1).*log(1-u1g);
      ap37=(g0a1-1).*log(v1g);
      ap3=ap31+sum(ap32+ap33)-ap34-ap35-sum(ap36(1:K1)+ap37(1:K1));
      
      ap41=betainc(ibetax1, lambda1g, N1bar-lambda1g);
      ap42=betainc(ibetax2, lambda1g, N1bar-lambda1g);
      ap43=gammainc(b1.*igammax1, g0a1);
      ap44=gammainc(b1.*igammax2, g0a1);
      ap40=log(ap41-ap42)+log(ap43-ap44);
      ap4=sum(ap40(1:K1));      
      
      ap5=b*g0alpha+sum(b1(1:K1).*v1g(1:K1))-sum(b1.*g0alpha1);
      
      ap=priorr*exp(ap1+ap2+ap3+ap4+ap5);
     %---------------------------------------------------------------------
     % End of calculating ap;
        if ap==0; A=1; else A=min(1, 1/ap); end
        
        if rand < A %deleting g from XG1;
            XG1(g0,:)=[]; a1g(g0,:)=[]; p1g1(g0,:)=[]; alpha1g1(g0,:)=[];
            n_G1=n_G1-1; Z2(i, :)=[i, g, 1, 2, -1];
            
            XG2=[XG2; Xg]; ag=[ag; g0a]; n_G2=n_G2+1; G2(i)=g; 
            pg1=[pg1; g0p]; alphag1=[alphag1; g0alpha]; 
            
        else G1(i)=g; Z2(i, :)=[i, g, 1, 1, 0];
        end
   %-----------------------------------------------------------------------
   %End of deleting from XG1;
        
    else %group==2 adding a gene to XG1 from XG2?
        g0=randsample(n_G2, 1); % choose one gene for adding;       
        Xg=XG2(g0, :); Xg0=Xg(1:n);  g=Xg(n+1);
        n1=1; n2=n0(1); X1g0=cell(1,K); g0a1=[];
        for k=1:K; g0a1(k)=sum(Xg0(n1:n2));
            if k<K; n1=n1+n0(k); n2=n2+n0(k+1); end;
        end

        % Calculating Acceptance Probability: ap
        %----------------------------------------------------------------
        g0p=pg1(g0); g0alpha=alphag1(g0); g0a=ag(g0); lambda1g=g0a1./b1; 
        
        u1g=zeros(1,K); v1g=zeros(1,K);
        ibetax1=min(1, K*g0p/K1); 
        ibetax2=max(0, (K*g0p-1)/K1);
        igammax1=min(N1bar, N1bar*(K*g0alpha/(K1*Nbar)));
        igammax2=max(0, N1bar*((K*g0alpha/Nbar-1)/K1));

        for k1=1:K
            u1g(k1)=ibetax1;
          while u1g(k1) >= ibetax1 | u1g(k1) <= ibetax2; 
             u1g(k1)=betarnd(lambda1g(k1), N1bar(k1)-lambda1g(k1));
          end;
            v1g(k1)=igammax1(k1);
          while v1g(k1) >= igammax1(k1) | v1g(k1) <= igammax2(k1)
             v1g(k1)=gamrnd(g0a1(k1), 1./b1(k1));     
          end;
        end
        
        g0p1=u1g; g0alpha1=v1g; von=v1g./N1bar; 
        g0p1(K)=K*g0p-sum(u1g(1:K1));
        g0alpha1(K)=N1bar(K)*(K*g0alpha/Nbar-sum(von(1:K1)));
        
      ap11=sum(g0a1.*log(g0p1))-g0a*log(g0p);
      ap12=sumN*g0p-sum(sumN1.*g0p1);
      ap1=2*log(K)+ap11+ap12;

      ap21=log(N1bar(K))+sum(gammaln(t*N1bar))+gammaln(g0a);
      ap22=gammaln(t*g0alpha)+gammaln(t*(Nbar-g0alpha))+...
            log(gammainc(b*Nbar, g0a));
      ap23=gammaln(lambda1g)+gammaln(N1bar-lambda1g);
      ap24=log(Nbar)+gammaln(t*Nbar)+gammaln(g0a1(K));
      ap25=gammaln(t*g0alpha1)+gammaln(t*(N1bar-g0alpha1))+...
           log(gammainc(b1.*N1bar, g0a1));
      ap26=gammaln(N1bar(1:K1));
      ap2=ap21+ap22+sum(ap23(1:K1))-ap24-sum(ap25)-sum(ap26);
      
      ap31=g0a1(K)*log(b1(K));
      ap32=(g0a1-1).*log(g0alpha1)+(t*g0alpha1-1).*log(g0p1);
      ap33=(t*(N1bar-g0alpha1)-1).*log(1-g0p1);
      ap34=g0a*log(b)+(g0a-1)*log(g0alpha);
      ap35=(t*g0alpha-1)*log(g0p)+(t*(Nbar-g0alpha)-1)*log(1-g0p);
      ap36=(lambda1g-1).*log(u1g)+(N1bar-lambda1g-1).*log(1-u1g);
      ap37=(g0a1-1).*log(v1g);
      ap3=ap31+sum(ap32+ap33)-ap34-ap35-sum(ap36(1:K1)+ap37(1:K1));
      
      ap41=betainc(ibetax1, lambda1g, N1bar-lambda1g);
      ap42=betainc(ibetax2, lambda1g, N1bar-lambda1g);
      ap43=gammainc(b1.*igammax1, g0a1);
      ap44=gammainc(b1.*igammax2, g0a1);
      ap40=log(ap41-ap42)+log(ap43-ap44);
      ap4=sum(ap40(1:K1));      
      
      ap5=b*g0alpha+sum(b1(1:K1).*v1g(1:K1))-sum(b1.*g0alpha1);
      
      ap=priorr*exp(ap1+ap2+ap3+ap4+ap5);
     %---------------------------------------------------------------------
     % End of calculating ap;
       if rand < ap %adding g to XG1;
           XG1=[XG1; Xg]; n_G1=n_G1+1; a1g=[a1g; g0a1]; 
           p1g1=[p1g1; g0p1]; alpha1g1=[alpha1g1; g0alpha1]; 
            
           XG2(g0, :)=[]; ag(g0)=[]; pg1(g0)=[]; alphag1(g0)=[]; 
           n_G2=n_G2-1; G1(i)=g; Z2(i, :)=[i, g, 2, 1, 1];
        else G2(i)=g; Z2(i, :)=[i, g, 2, 2, 0];
       end
       %-----------------------------------------------------------------
    %End of adding to XG1 from XG2;             
    end %end of if group==1 (end of deleting or adding);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    p1g0=p1g1; pg0=pg1; alpha1g0=alpha1g1; alphag0=alphag1;
    
    Z1(i+1, :)=[i, n_G1, n_G2];
        
end %end of for i=1:I;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  END of iteration                                                       %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for i=1:G
    y=0;
    aa=find(Z2(:, 2)==i);
    ab=Z2(aa, :);
    ac=find(ab(:, 5)>0 | ab(:, 5)<0);
    ad=ab(ac, :);
    adlen=size(ad, 1);
    if adlen==0
        if ab(1, 3)==1; y=I; else y=0; 
        end
    elseif adlen==1   
        if ad(1, 3)==1; y=ad(1,1)-1; else y=I-ad(1,1)+1; end;
    elseif adlen > 1
        if ad(1, 3)==1; y=ad(1,1)-1; else y=1; end;
    for j=2:adlen
        if ad(j, 3)==1; y=y+ad(j, 1)-ad(j-1, 1); end
    end
    if ad(adlen, 3)==2; y=y+I-ad(adlen, 1); end
    end
    y1(i)=y;
end

y2=I-y1; y3=y1'./y2'; y4=y3/priorr; y5=y1/I;
z=[(1:G)', y5', y4];
z0=sortrows(z, 3);
for i=1:G
    z(i, :)=z0(G-i+1, :);
end
Z=Z2;
