function [X, n0] = simuK2(pg, n0, N, seed)
%
%Simulate K=2 SAGE data:
%
%[X, n0] = simuK2(pg, n0, N, seed) simulate SAGE data set from intensity pg
%generated from real data proportions or your own generated proportions.
%All input parameters have default values: 
%        pg=pg2, a two column matrix for which top 50 BW values for 2 group
%           are different proportion. Others are same.
%        n0=[3,3] means each group has 3 libraries.
%        N = SX2*10, a vector with the length length(N) = sum(n0), gives
%            the total counts for each library.
%        seed=234 for random seed or state.
%SX2 is the vector containning the summation of each library counts for
%real data sagedata.txt. pg2 and SX2 will be downloaded to the same 
%directory with the programs.
%
%--------------------------------------------------------------------------
%Output arguments:
%   X: simulated matrix with the same rows as pg and the same column as N,
%      in addition, sum(X, 1) = N.
%   n0: = input n0.
%
% References:
%
%   [1] Wang, Z. and Lin, S. Modeling and Analysis of SAGE Libraries,
%       manuscript at http://www.stat.ohio-state.edu/~statgen/DE-
%       SAGE.html, 2005.

%Author: Zailong Wang & Shili Lin, MBI at The Ohio State University
%$Revision: 1.5 $  $Date: 2005/10/12 21:49:12 $

if nargin < 4, seed = 234; end
if nargin < 3, N = load('SX2')*10; end
if nargin < 2, n0=[3,3]; end
if nargin < 1, pg = load('pg2'); end

K=2; initstate(seed);
G=size(pg, 1); %total genes (rows)

n=sum(n0); %individuals (columns)
dei=find(pg(:,1)<pg(:,2)); %deferential expression index.
pde=1-sum(pg);
a1=sum(pg(dei,:));
weit=(a1+pde)./a1;

p=pg; I=length(dei);
for i=1:I    
    j=dei(i);
    [i,j];
        p(j,:)=pg(j,:).*weit;
end

X=zeros(G, n);

n1=1; n2=n0(1);
for k=1:K
    for j=n1:n2;
        X(:, j)=multrnd(N(j), p(:, k), 1);
    end
    if k<K
        n1=n2+1; n2=n2+n0(k+1);
    end
end
