function [X, n0] = simuK3(pg, n0, N, seed)
%
%Simulate K=3 SAGE data:
%
%[X, n0] = simuK3(pg, n0, N, seed) simulate SAGE data set from intensity pg
%generated from real data proportions or your own generated proportions.
%All input parameters have default values: 
%        pg=pg3, a three column matrix for which 100 values for 3 group
%           are different proportion. Others are same.
%        n0=[3,3,3] means each group has 3 libraries.
%        N = [SX2*10, 30000, 30000, 30000], a vector with the length 
%           length(N) = sum(n0), gives the total counts for each library.           
%        seed=234 for random seed or state.
%SX2 is the vector containning the summation of each library counts for
%real data sagedata.txt. pg3 and SX2 will be downloaded to the same 
%directory with the programs.
%
%--------------------------------------------------------------------------
%Output arguments:
%   X: simulated matrix with the same rows as pg and the same column as N,
%      in addition, sum(X, 1)=N.
%   n0: = input n0.
%
% References:
%
%   [1] Wang, Z. and Lin, S. Modeling and Analysis of SAGE Libraries,
%       manuscript at http://www.stat.ohio-state.edu/~statgen/DE-
%       SAGE.html, 2005.

%Author: Zailong Wang & Shili Lin, MBI at The Ohio State University
%$Revision: 1.5 $  $Date: 2005/10/12 21:49:12 $

if nargin < 4, seed = 234; end
if nargin < 3, load('SX2');
    N = [SX2*10, 30000, 30000, 30000]; 
end
if nargin < 2, n0=[3,3,3]; end
if nargin < 1, pg = load('pg3'); end
%--------------------------------------------------------------%

K=length(n0); initstate(seed)
G=size(pg, 1); %total genes (rows)
n=sum(n0); %individuals (columns)

dei=find(pg(:,1)<pg(:,2)); %deferential expression index.
pde=1-sum(pg(:,[1,2]));
a1=sum(pg(dei,[1,2]));
weit=(a1+pde)./a1;

p=pg; I=length(dei);
for i=1:I    
    j=dei(i);
    p(j,[1,2])=pg(j,[1,2]).*weit;
end

%Normalize p_{3g} (column 3).
dei3=find(pg(:,1)~=pg(:,3)); %deferential expression index.
pde3=1-sum(pg(:,3));
a3=sum(pg(dei3,3));
weit3=(a3+pde3)./a3;

I=length(dei3);
for i=1:I    
    j=dei3(i);
        p(j,3)=pg(j,3)*weit3;
end

X=zeros(G, n);

n1=1; n2=n0(1);
for k=1:K
    for j=n1:n2
        X(:, j)=multrnd(N(j), p(:, k), 1);
    end
    if k<K
        n1=n2+1; n2=n2+n0(k+1);
    end
end

